@@ -13,23 +13,31 @@
 #include <QDebug>
 #include <QDesktopServices>
 #include <QErrorMessage>
+#include <QFileDialog>
 #include <QInputDialog>
 #include <QListWidgetItem>
 #include <QMessage>
 #include <QMessageAddress>
 #include <QMessageBox>
 #include <QPixmap>
+#include <QProcess>
 #include <QSettings>
 #include <QStandardItem>
+#include <QStatusBar>
 #include <QtDebug>
+#include <QTimer>
 #include <QUrl>
 
 PanicButtonMainWindow::PanicButtonMainWindow(QWidget *parent) :
-        QMainWindow(parent),
-        ui(new Ui::PanicButtonMainWindow),
-        allContactsIndex(0),
-        emergencyContactsIndex(0),
-        gpsUpdateIntervalSet(false)
+    QMainWindow(parent),
+    ui(new Ui::PanicButtonMainWindow),
+    allContactsIndex(0),
+    emergencyContactsIndex(0),
+    gpsUpdateIntervalSet(false),
+    runApplicationOnPanic(false),
+    runThisApplicationOnPanic(""),
+    writeDefaultSettings(false),
+    allContactsRemoveRequest(0)
 {
     ui->setupUi(this);
     ui->panicButton->setStyleSheet("color:green");
@@ -42,119 +50,86 @@ PanicButtonMainWindow::PanicButtonMainWindow(QWidget *parent) :
     emergencyContacts = settings.value("Emergency Contacts").toMap();
     panicMessage = settings.value("Panic Message", tr("This is my location")).toString();
     gpsUpdateInterval = settings.value("GPS Update Interval", 60).toInt();
+    deleteAllContactsOnPanic = settings.value("Delete Contacts on Panic", "false").toBool();
+    runApplicationOnPanic = settings.value("Run Application On Panic", "false").toBool();
+    runThisApplicationOnPanic = settings.value("Run This Application On Panic", "").toString();
 
-    menuBar = new QMenuBar();
-    aboutAction = new QAction(tr("About Panic Button"), this);
-    connect(aboutAction, SIGNAL(triggered()), this, SLOT(about()));
-    menuBar->addAction(aboutAction);
+    ui->actionDelete_All_Contacts_On_Panic->setChecked(deleteAllContactsOnPanic);
 
-    // TODO: Disable this entry until we have updated emergencyContacts.
-    // TODO: Leave the entry enabled and if it's selected before
-    // the updates arrive, ask if the users wants to cancel the
-    // update.
-    setContactsAction = new QAction(tr("Choose Contacts"), this);
-    connect(setContactsAction, SIGNAL(triggered()), this, SLOT(setContacts()));
-    menuBar->addAction(setContactsAction);
+    ui->actionRun_Application_On_Panic->setEnabled(false);
 
-    clearContactsAction = new QAction(tr("Clear Contacts"), this);
-    connect(clearContactsAction, SIGNAL(triggered()), this, SLOT(clearContacts()));
-    menuBar->addAction(clearContactsAction);
+    /*
+      Don't invoke the application selection dialog in
+      on_actionRun_Application_On_Panic_toggled() at
+      startup time because that would confuse the user.
+      The dialog will appear before the panic button and
+      the menu.
 
-    setPanicMessageAction = new QAction(tr("Set Panic Message"), this);
-    connect(setPanicMessageAction, SIGNAL(triggered()), this, SLOT(setPanicMessage()));
-    menuBar->addAction(setPanicMessageAction);
+      To accomplish this, make the value of runApplicationOnPanic
+      depend on the non-empty value of runThisApplicationOnPanic.
+    */
+    if (runThisApplicationOnPanic.isEmpty())
+        runApplicationOnPanic = false;
 
-    setGPSUpdateIntervalAction = new QAction(tr("Set GPS Update Interval"), this);
-    connect(setGPSUpdateIntervalAction, SIGNAL(triggered()), this, SLOT(setGPSUpdateInterval()));
-    menuBar->addAction(setGPSUpdateIntervalAction);
+    ui->actionRun_Application_On_Panic->setChecked(runApplicationOnPanic);
 
-    helpAction = new QAction(tr("Help"), this);
-    connect(helpAction, SIGNAL(triggered()), this, SLOT(help()));
-    menuBar->addAction(helpAction);
-
-    setMenuBar(menuBar);
-
-    service = new QMessageService(this);
-    if (service) {
-        connect(service, SIGNAL(stateChanged(QMessageService::State)),
-                this, SLOT(messageServiceStateChanged(QMessageService::State)));
+    messageService = new QMessageService(this);
+    if (messageService) {
+        connect(messageService, SIGNAL(stateChanged(QMessageService::State)),
+                this, SLOT(messageService_stateChanged(QMessageService::State)));
         messageServiceReady = true;
     }
 
-    source  = QGeoPositionInfoSource::createDefaultSource(this);
-    // TODO: schedule another attempt to createDefaultSource()
-    // if this one fails.
-    //
-    // TODO: Display a yellow banner "Getting GPS fix"
-    // until the first time positionUpdated() is called.
-    if (source) {
-        connect(source, SIGNAL(positionUpdated(QGeoPositionInfo)),
-                this, SLOT(positionUpdated(QGeoPositionInfo)));
-        source->startUpdates();
-    }
-
-    connect(&chooserDialog, SIGNAL(accepted()), this, SLOT(accept()));
-    connect(&chooserDialog, SIGNAL(rejected()), this, SLOT(reject()));
+    connect(&chooserDialog, SIGNAL(accepted()), this, SLOT(chooserDialog_accepted()));
+    connect(&chooserDialog, SIGNAL(rejected()), this, SLOT(chooserDialog_rejected()));
 
     connect(&allContactsFetchRequest, SIGNAL(stateChanged(QContactAbstractRequest::State)),
-            this, SLOT(allContactsFetchRequestStateChanged(QContactAbstractRequest::State)));
+            this, SLOT(allContactsFetchRequest_stateChanged(QContactAbstractRequest::State)));
     connect(&allContactsFetchRequest, SIGNAL(resultsAvailable()),
-            this, SLOT(allContactsFetchRequestResultsAvailable()));
+            this, SLOT(allContactsFetchRequest_resultsAvailable()));
+
     allContactsFetchRequest.setManager(&cm);
 
     connect(&emergencyContactsFetchRequest, SIGNAL(stateChanged(QContactAbstractRequest::State)),
-            this, SLOT(emergencyContactsFetchRequestStateChanged(QContactAbstractRequest::State)));
+            this, SLOT(emergencyContactsFetchRequest_stateChanged(QContactAbstractRequest::State)));
     connect(&emergencyContactsFetchRequest, SIGNAL(resultsAvailable()),
-            this, SLOT(emergencyContactsFetchRequestUpdateResultsAvailable()));
+            this, SLOT(emergencyContactsFetchRequestUpdate_resultsAvailable()));
 
     emergencyContactsFetchRequest.setManager(&cm);
+    QTimer::singleShot(1000, this, SLOT(contactGPS()));
+    QTimer::singleShot(1000, this, SLOT(fetchEmergencyContacts()));
 
-    if (buildEmergencyContactsFetchRequest()) {
-        // TODO: Display a yellow banner "Getting stored emergency contacts from contact manager"
-        // until emergencyContactsFetchRequestStateChanged() is called with
-        // QContactAbstractRequest::FinishedState
-        emergencyContactsFetchRequest.start();
-    }
-
+    if (deleteAllContactsOnPanic)
+        QTimer::singleShot(1000, this, SLOT(createAllContactsRemoveRequest()));
 }
 
+
 PanicButtonMainWindow::~PanicButtonMainWindow()
 {
 
     QSettings settings("Whatever", "PanicButton", this);
+
+    // TODO: Constants for the default settings.
+    if (writeDefaultSettings) {
+        emergencyContacts.clear();
+        panicMessage = tr("This is my location");
+        gpsUpdateInterval = 60;
+        deleteAllContactsOnPanic = false;
+        runApplicationOnPanic = false;
+        runThisApplicationOnPanic = "";
+    }
+
     settings.setValue("Emergency Contacts", emergencyContacts);
     settings.setValue("Panic Message", panicMessage);
     settings.setValue("GPS Update Interval", gpsUpdateInterval);
+    settings.setValue("Delete Contacts on Panic", deleteAllContactsOnPanic);
+    settings.setValue("Run Application On Panic", runApplicationOnPanic);
+    settings.setValue("Run This Application On Panic", runThisApplicationOnPanic);
 
     delete ui;
-
-    if (service)
-        delete service;
-
-    if (source)
-        delete source;
-}
-
-void PanicButtonMainWindow::about() {
-    QMessageBox::about(this,
-                       tr("About Panic Button"),
-                       tr("<h1>Panic Button &copy; 2010 by David Talmage</h1><p>Version 0.3</p>\
-                          <p>Send your location to your important people when you press the Panic Button.</p>"));
-}
-
-void PanicButtonMainWindow::accept()
-{
-    emergencyContacts.clear();
-    QList<QStandardItem *> newEmergencyContacts = chooserDialog.selectedItems();
-    foreach(QStandardItem *item, newEmergencyContacts) {
-        QString guid = item->data(Qt::UserRole).toString();
-        qDebug() << "accept(): Adding " << guid << ":"
-                << allContacts.value(guid) << " to emergencyContacts";
-        emergencyContacts.insert(guid, allContacts.value(guid));
-    }
-
-    chooserDialog.clear();
-    allContacts.clear();
+    delete messageService;
+    delete source;
+    delete allContactsRemoveRequest;
 }
 
 void PanicButtonMainWindow::addToChooserDialog(QContactFetchRequest &request)
@@ -192,33 +167,35 @@ void PanicButtonMainWindow::addToChooserDialog(QContactFetchRequest &request)
 void PanicButtonMainWindow::addToEmergencyContacts(QContactFetchRequest &request)
 {
     QList<QContact> contacts = request.contacts();
-//    qDebug()<< "emergencyContacts: some results!";
+    //    qDebug()<< "emergencyContacts: some results!";
     for(; emergencyContactsIndex < contacts.length(); emergencyContactsIndex++) {
         QContactGuid guid = contacts.at(emergencyContactsIndex).detail<QContactGuid>();
         QList<QContactPhoneNumber> numbers =
                 contacts.at(emergencyContactsIndex).details<QContactPhoneNumber>();
 
         qDebug() << "addToEmergencyContacts(): Retrieved contact:" << guid << " "
-                << contacts.at(emergencyContactsIndex).displayLabel();
+                 << contacts.at(emergencyContactsIndex).displayLabel();
 
         // Choose only mobile numbers in numbers.
         foreach(QContactPhoneNumber number, numbers) {
             if (number.subTypes().contains(QContactPhoneNumber::SubTypeMobile)) {
                 emergencyContacts.insert(guid.guid(), number.number());
                 qDebug() << "Mobile number:" << guid << " " << number;
-          }
+            }
         }
     }
 }
 
-void PanicButtonMainWindow::allContactsFetchRequestResultsAvailable()
+void PanicButtonMainWindow::allContactsFetchRequest_resultsAvailable()
 {
     QContactFetchRequest *request = qobject_cast<QContactFetchRequest*>(QObject::sender());
-//    qDebug()<< "allContacts: some results!";
+    //    qDebug()<< "allContacts: some results!";
+
+     // TODO: QProgressDialog
     addToChooserDialog(*request);
 }
 
-void PanicButtonMainWindow::allContactsFetchRequestStateChanged(QContactAbstractRequest::State newState) {
+void PanicButtonMainWindow::allContactsFetchRequest_stateChanged(QContactAbstractRequest::State newState) {
     QContactFetchRequest *request = qobject_cast<QContactFetchRequest*>(QObject::sender());
 
     // TALMAGE: This might not be the right way to handle
@@ -244,14 +221,8 @@ void PanicButtonMainWindow::allContactsFetchRequestStateChanged(QContactAbstract
 
 }
 
-void PanicButtonMainWindow::clearContacts()
+void PanicButtonMainWindow::allContactsRemoveRequest_stateChanged(QContactAbstractRequest::State)
 {
-    // Require permission before clearing.
-    int r = QMessageBox::warning(this, tr("Confirm"),
-                                 tr("Clear emergency contacts?"),
-                                 QMessageBox::Yes | QMessageBox::No);
-    if (QMessageBox::Yes == r)
-        emergencyContacts.clear();
 }
 
 bool PanicButtonMainWindow::buildEmergencyContactsFetchRequest()
@@ -297,10 +268,63 @@ bool PanicButtonMainWindow::buildEmergencyContactsFetchRequest()
     return answer;
 }
 
-void PanicButtonMainWindow::emergencyContactsFetchRequestUpdateResultsAvailable()
+void PanicButtonMainWindow::chooserDialog_accepted()
+{
+    emergencyContacts.clear();
+    QList<QStandardItem *> newEmergencyContacts = chooserDialog.selectedItems();
+    foreach(QStandardItem *item, newEmergencyContacts) {
+        QString guid = item->data(Qt::UserRole).toString();
+        qDebug() << "accept(): Adding " << guid << ":"
+                 << allContacts.value(guid) << " to emergencyContacts";
+        emergencyContacts.insert(guid, allContacts.value(guid));
+    }
+
+    chooserDialog.clear();
+    allContacts.clear();
+}
+
+void PanicButtonMainWindow::chooserDialog_rejected()
+{
+    // No change to emergencyContacts
+    chooserDialog.clear();
+    allContacts.clear();
+}
+
+void PanicButtonMainWindow::contactGPS()
+{
+    source  = QGeoPositionInfoSource::createDefaultSource(this);
+    // TODO: schedule another attempt to createDefaultSource()
+    // if this one fails.
+    //
+    // TODO: Display a yellow banner "Getting GPS fix"
+    // until the first time positionUpdated() is called.
+    if (source) {
+        connect(source, SIGNAL(positionUpdated(QGeoPositionInfo)),
+                this, SLOT(source_positionUpdated(QGeoPositionInfo)));
+        source->startUpdates();
+    }
+}
+
+void PanicButtonMainWindow::createAllContactsRemoveRequest()
+{
+    delete allContactsRemoveRequest;
+
+    // TODO: Request contactIds asynchronously
+    // TODO: QProgressDialog
+    allContactsRemoveRequest = new QContactRemoveRequest(this);
+    allContactsRemoveRequest->setContactIds(cm.contactIds());
+    allContactsRemoveRequest->setManager(&cm);
+
+    //    connect(allContactsRemoveRequest, SIGNAL(stateChanged(QContactAbstractRequest::State)),
+    //            this, SLOT(allContactsRemoveRequestStateChanged(QContactAbstractRequest::State)));
+}
+
+void PanicButtonMainWindow::emergencyContactsFetchRequestUpdate_resultsAvailable()
 {
     QContactFetchRequest *request = qobject_cast<QContactFetchRequest*>(QObject::sender());
 
+    // TODO: QProgressDialog
+
     // TALMAGE: Must I check error() here?  Is it OK to assume that
     // error() == QContactManager::NoError will be handed to
     // emergencyContactsFetchRequestStateChanged() with
@@ -308,7 +332,7 @@ void PanicButtonMainWindow::emergencyContactsFetchRequestUpdateResultsAvailable(
     addToEmergencyContacts(*request);
 }
 
-void PanicButtonMainWindow::emergencyContactsFetchRequestStateChanged(QContactAbstractRequest::State newState) {
+void PanicButtonMainWindow::emergencyContactsFetchRequest_stateChanged(QContactAbstractRequest::State newState) {
     QContactFetchRequest *request = qobject_cast<QContactFetchRequest*>(QObject::sender());
 
     // TALMAGE: This might not be the right way to handle
@@ -321,25 +345,22 @@ void PanicButtonMainWindow::emergencyContactsFetchRequestStateChanged(QContactAb
 
         addToEmergencyContacts(*request);
         emergencyContactsIndex = 0;
-
     } else if (newState == QContactAbstractRequest::CanceledState) {
         qDebug() << "Fetch operation canceled!";
     }
 }
 
-void PanicButtonMainWindow::help()
+void PanicButtonMainWindow::fetchEmergencyContacts()
 {
-    // Help me, Spock!
-    // TODO: Discover the documentation in a platform independent way.
-    // TODO: Use QtAssistant to display the documentation.
-    QUrl url("/opt/usr/share/panicbutton/panicbutton.html");
-    url.setScheme(("file"));
-    qDebug() << "help(): " << url;
-    bool answer = QDesktopServices::openUrl(url);
-    qDebug() << "help(): openUrl returned " << answer;
+    if (buildEmergencyContactsFetchRequest()) {
+        // TODO: Display a yellow banner "Getting stored emergency contacts from contact manager"
+        // until emergencyContactsFetchRequestStateChanged() is called with
+        // QContactAbstractRequest::FinishedState
+        emergencyContactsFetchRequest.start();
+    }
 }
 
-void PanicButtonMainWindow::messageServiceStateChanged(QMessageService::State newState)
+void PanicButtonMainWindow::messageService_stateChanged(QMessageService::State newState)
 {
     qDebug() << "messageServiceStateChanged() " << newState;
 
@@ -351,10 +372,112 @@ void PanicButtonMainWindow::messageServiceStateChanged(QMessageService::State ne
     }
 }
 
+void PanicButtonMainWindow::on_actionAbout_Panic_Button_triggered() {
+    QMessageBox::about(this,
+                       tr("About Panic Button"),
+                       tr("<h1>Panic Button &copy; 2011 by David Talmage</h1><p>Version 0.4</p>\
+                          <p>Send your location to your important people when you press the Panic Button.</p>\
+                          <p>Distribution of this free software is governed by the GNU Public License version x.y.</p>\
+                          <p>Provided AS IS with NO WARRANTY OF ANY KIND."));
+}
+
+void PanicButtonMainWindow::on_actionClear_Contacts_triggered()
+{
+    // Require permission before clearing.
+    int r = QMessageBox::warning(this, tr("Confirm"),
+                                 tr("Clear emergency contacts?"),
+                                 QMessageBox::Yes | QMessageBox::No);
+    if (QMessageBox::Yes == r)
+        emergencyContacts.clear();
+}
+
+void PanicButtonMainWindow::on_actionHelp_triggered()
+{
+    // Help me, Spock!
+    // TODO: Discover the documentation in a platform independent way.
+    // TODO: Use QtAssistant to display the documentation.
+    QUrl url("/opt/usr/share/panicbutton/panicbutton.html");
+    url.setScheme(("file"));
+    qDebug() << "help(): " << url;
+    bool answer = QDesktopServices::openUrl(url);
+    qDebug() << "help(): openUrl returned " << answer;
+}
+
+// TODO: Display a yellow banner "Getting contacts from contact manager"
+// until allContactsFetchRequestStateChanged() is called with
+// QContactAbstractRequest::FinishedState
+void PanicButtonMainWindow::on_actionChoose_Contacts_triggered()
+{
+    chooserDialog.clear();
+    allContactsIndex = 0;
+    allContactsFetchRequest.start();
+}
+
+void PanicButtonMainWindow::on_actionDelete_All_Contacts_On_Panic_toggled(bool checked)
+{
+    deleteAllContactsOnPanic = checked;
+
+    if (deleteAllContactsOnPanic) {
+        createAllContactsRemoveRequest();
+    } else {
+        delete allContactsRemoveRequest;
+        allContactsRemoveRequest = 0;
+    }
+}
+
+/*
+  Turn the Run Application On Panic feature on or off.
+  When turning the feature on, display a QFileDialog that asks
+  for the name of the program to run.  The user can press Cancel
+  to retain the previously selected application.
+*/
+void PanicButtonMainWindow::on_actionRun_Application_On_Panic_toggled(bool checked)
+{
+    runApplicationOnPanic = checked;
+
+    // BUG: On Maemo you can't walk the full
+    // directory hierarchy.  You're limited to /home/user
+    // and the memory card.
+    if (checked) {
+        QString startDirectory = QDir(runThisApplicationOnPanic.isEmpty() ? "/" : runThisApplicationOnPanic).absolutePath();
+        QString newApp = QFileDialog::getOpenFileName(this,
+                                                      tr("Choose an application"),
+                                                      startDirectory);
+        if (! newApp.isEmpty())
+            runThisApplicationOnPanic = newApp;
+    }
+}
+
+void PanicButtonMainWindow::on_actionSet_GPS_Update_Interval_triggered()
+{
+    bool ok;
+    int newInterval = QInputDialog::getInteger(this, tr("GPS Update Interval"),
+                                               tr("Seconds between location reports from the GPS"),
+                                               gpsUpdateInterval, 1, INT_MAX/1000, 1, &ok);
+    if (ok) {
+        gpsUpdateInterval = newInterval;
+        // The arg is milliseconds, but gpsUpdateInterval is seconds,
+        // so multiply by 1000.
+        source->setUpdateInterval(gpsUpdateInterval * 1000);
+        gpsUpdateIntervalSet = true;
+    }
+}
+
+void PanicButtonMainWindow::on_actionSet_Panic_Message_triggered()
+{
+    QString newPanicMessage = QInputDialog::getText(this,
+                                                    tr("Panic Message"),
+                                                    tr("What do you want to say?"),
+                                                    QLineEdit::Normal,
+                                                    panicMessage);
+    if (0 != newPanicMessage)
+        panicMessage = newPanicMessage;
+}
+
 void PanicButtonMainWindow::on_panicButton_clicked()
 {
     QString completePanicMessage = QString("%1 %2").arg(panicMessage).arg(location.coordinate().toString());
-    work *w = new work::work();
+    work *w = new work();
 
     w->message = completePanicMessage;
     w->recipients = phoneNumbers();
@@ -363,6 +486,28 @@ void PanicButtonMainWindow::on_panicButton_clicked()
         ui->panicButton->setText(tr("Panic!"));
         workQueue.append(w);
         sendMessage();
+
+        if (deleteAllContactsOnPanic) {
+            /*
+              Start it up and forget about it.
+              We neither want nor need notification
+              about the state of the request.
+              */
+            allContactsRemoveRequest->start();
+            writeDefaultSettings = true;
+            /*
+            delete allContactsRemoveRequest;
+            allContactsRemoveRequest = 0;
+            */
+        }
+
+        if (runApplicationOnPanic &&
+                (runThisApplicationOnPanic.length() > 0)) {
+            // New, independent process.
+            // If it starts, it persists even if Panic Button exits first.
+            QStringList args;
+            bool success = QProcess::startDetached(runThisApplicationOnPanic, args, QDir::homePath());
+        }
     } else {
         QMessageBox::information(this, tr("Choose a contact first"), tr("No one to send to!"));
         qDebug() << "No one to send to!";
@@ -373,9 +518,18 @@ QList<QVariant> PanicButtonMainWindow::phoneNumbers() {
     return emergencyContacts.values();
 }
 
+void PanicButtonMainWindow::printEmergencyContacts()
+{
+    qDebug() << "Emergency Contacts";
+    foreach(QString key, emergencyContacts.keys()) {
+        qDebug() << "emergencyContact guid: " << key
+                 << " phone: " << emergencyContacts.value(key);
+    }
+}
+
 // TODO: Display a yellow banner "Getting GPS fix"
 // until the first time positionUpdated() is called.
-void PanicButtonMainWindow::positionUpdated(const QGeoPositionInfo &info)
+void PanicButtonMainWindow::source_positionUpdated(const QGeoPositionInfo &info)
 {
     location = info;
     if (!gpsUpdateIntervalSet) {
@@ -384,22 +538,6 @@ void PanicButtonMainWindow::positionUpdated(const QGeoPositionInfo &info)
     }
 }
 
-void PanicButtonMainWindow::printEmergencyContacts()
-{
-    qDebug() << "Emergency Contacts";
-    foreach(QString key, emergencyContacts.keys()) {
-        qDebug() << "emergencyContact guid: " << key
-                << " phone: " << emergencyContacts.value(key);
-    }
-}
-
-void PanicButtonMainWindow::reject()
-{
-    // No change to emergencyContacts
-    chooserDialog.clear();
-    allContacts.clear();
-}
-
 /*
     Use the QMessageService to send one message from the
     work queue.
@@ -452,7 +590,7 @@ void PanicButtonMainWindow::sendMessage()
                                              QMessageBox::Yes | QMessageBox::No);
                 if (QMessageBox::Yes == r)
 #endif
-                service->send(message);
+                    messageService->send(message);
                 // TALMAGE: Given the way that w->recipients is constructed,
                 // must I delete each of its QStrings?
             }
@@ -463,38 +601,3 @@ void PanicButtonMainWindow::sendMessage()
     }
 }
 
-// TODO: Display a yellow banner "Getting contacts from contact manager"
-// until allContactsFetchRequestStateChanged() is called with
-// QContactAbstractRequest::FinishedState
-void PanicButtonMainWindow::setContacts()
-{
-    chooserDialog.clear();
-    allContactsIndex = 0;
-    allContactsFetchRequest.start();
-}
-
-void PanicButtonMainWindow::setGPSUpdateInterval()
-{
-    bool ok;
-    int newInterval = QInputDialog::getInteger(this, tr("GPS Update Interval"),
-                                               tr("Seconds between location reports from the GPS"),
-                                               gpsUpdateInterval, 1, INT_MAX/1000, 1, &ok);
-    if (ok) {
-        gpsUpdateInterval = newInterval;
-        // The arg is milliseconds, but gpsUpdateInterval is seconds,
-        // so multiply by 1000.
-        source->setUpdateInterval(gpsUpdateInterval * 1000);
-        gpsUpdateIntervalSet = true;
-    }
-}
-
-void PanicButtonMainWindow::setPanicMessage()
-{
-    QString newPanicMessage = QInputDialog::getText(this,
-                                                    tr("Panic Message"),
-                                                    tr("What do you want to say?"),
-                                                    QLineEdit::Normal,
-                                                    panicMessage);
-    if (0 != newPanicMessage)
-        panicMessage = newPanicMessage;
-}
